/* Gpx Viewer 
 * Copyright (C) 2009-2009 Qball Cow <qball@sarine.nl>
 * Project homepage: http://blog.sarine.nl/ 

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gpx-parser.h>
#include <pango/pango.h>
#include <cairo.h>
#include <gdk/gdk.h>
#include <float.h>
#include <math.h>
#include <time.h>
#include <pango/pangocairo.h>
#include <stdlib.h>
#include <string.h>


#define GPX_TYPE_GRAPH (gpx_graph_get_type ())
#define GPX_GRAPH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GPX_TYPE_GRAPH, GpxGraph))
#define GPX_GRAPH_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GPX_TYPE_GRAPH, GpxGraphClass))
#define GPX_IS_GRAPH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GPX_TYPE_GRAPH))
#define GPX_IS_GRAPH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GPX_TYPE_GRAPH))
#define GPX_GRAPH_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GPX_TYPE_GRAPH, GpxGraphClass))

typedef struct _GpxGraph GpxGraph;
typedef struct _GpxGraphClass GpxGraphClass;
typedef struct _GpxGraphPrivate GpxGraphPrivate;

struct _GpxGraph {
	GtkEventBox parent_instance;
	GpxGraphPrivate * priv;
};

struct _GpxGraphClass {
	GtkEventBoxClass parent_class;
};

struct _GpxGraphPrivate {
	gint _smooth_factor;
	GpxTrack* track;
	PangoFontDescription* fd;
	cairo_surface_t* surf;
	gint LEFT_OFFSET;
	gint BOTTOM_OFFSET;
};


static gpointer gpx_graph_parent_class = NULL;

GType gpx_graph_get_type (void);
#define GPX_GRAPH_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPX_TYPE_GRAPH, GpxGraphPrivate))
enum  {
	GPX_GRAPH_DUMMY_PROPERTY,
	GPX_GRAPH_SMOOTH_FACTOR
};
static void gpx_graph_size_allocate_cb (GpxGraph* self, const GdkRectangle* alloc);
static void _gpx_graph_size_allocate_cb_gtk_widget_size_allocate (GtkWidget* _sender, const GdkRectangle* allocation, gpointer self);
static gboolean gpx_graph_button_press_event_cb (GpxGraph* self, const GdkEventButton* event);
static gboolean _gpx_graph_button_press_event_cb_gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self);
GpxGraph* gpx_graph_new (void);
GpxGraph* gpx_graph_construct (GType object_type);
GpxGraph* gpx_graph_new (void);
void gpx_graph_set_track (GpxGraph* self, GpxTrack* track);
static void gpx_graph_update_surface (GpxGraph* self, GtkWidget* win);
static gboolean gpx_graph_real_expose_event (GtkWidget* base, const GdkEventExpose* event);
gint gpx_graph_get_smooth_factor (GpxGraph* self);
void gpx_graph_set_smooth_factor (GpxGraph* self, gint value);
static void gpx_graph_finalize (GObject* obj);
static void gpx_graph_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void gpx_graph_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


static void g_cclosure_user_marshal_VOID__DOUBLE_DOUBLE (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);

static void _gpx_graph_size_allocate_cb_gtk_widget_size_allocate (GtkWidget* _sender, const GdkRectangle* allocation, gpointer self) {
	gpx_graph_size_allocate_cb (self, allocation);
}


static gboolean _gpx_graph_button_press_event_cb_gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self) {
	return gpx_graph_button_press_event_cb (self, event);
}


GpxGraph* gpx_graph_construct (GType object_type) {
	GpxGraph * self;
	PangoFontDescription* _tmp0_;
	self = g_object_newv (object_type, 0, NULL);
	_tmp0_ = NULL;
	self->priv->fd = (_tmp0_ = pango_font_description_new (), (self->priv->fd == NULL) ? NULL : (self->priv->fd = (pango_font_description_free (self->priv->fd), NULL)), _tmp0_);
	pango_font_description_set_family (self->priv->fd, "sans mono");
	g_object_set ((GtkWidget*) self, "app-paintable", TRUE, NULL);
	gtk_event_box_set_visible_window ((GtkEventBox*) self, TRUE);
	g_signal_connect_object ((GtkWidget*) self, "size-allocate", (GCallback) _gpx_graph_size_allocate_cb_gtk_widget_size_allocate, self, 0);
	g_signal_connect_object ((GtkWidget*) self, "button-press-event", (GCallback) _gpx_graph_button_press_event_cb_gtk_widget_button_press_event, self, 0);
	return self;
}


GpxGraph* gpx_graph_new (void) {
	return gpx_graph_construct (GPX_TYPE_GRAPH);
}


void gpx_graph_set_track (GpxGraph* self, GpxTrack* track) {
	GpxTrack* _tmp1_;
	GpxTrack* _tmp0_;
	cairo_surface_t* _tmp2_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->track = (_tmp1_ = (_tmp0_ = track, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->track == NULL) ? NULL : (self->priv->track = (g_object_unref (self->priv->track), NULL)), _tmp1_);
	/* Invalidate the previous plot, so it is redrawn */
	_tmp2_ = NULL;
	self->priv->surf = (_tmp2_ = NULL, (self->priv->surf == NULL) ? NULL : (self->priv->surf = (cairo_surface_destroy (self->priv->surf), NULL)), _tmp2_);
	/* Force a redraw */
	gtk_widget_queue_draw ((GtkWidget*) self);
}


/**
 * Private functions
 */
static gboolean gpx_graph_button_press_event_cb (GpxGraph* self, const GdkEventButton* event) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	if (self->priv->track == NULL) {
		result = FALSE;
		return result;
	}
	_tmp0_ = FALSE;
	if ((*event).x > self->priv->LEFT_OFFSET) {
		_tmp0_ = (*event).x < (((GtkWidget*) self)->allocation.width - 10);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		double elapsed_time;
		time_t time;
		GList* iter;
		elapsed_time = (double) gpx_track_get_total_time (self->priv->track);
		time = (time_t) ((((*event).x - self->priv->LEFT_OFFSET) / ((((GtkWidget*) self)->allocation.width - 10) - self->priv->LEFT_OFFSET)) * elapsed_time);
		iter = g_list_first (self->priv->track->points);
		time = time + gpx_point_get_time ((GpxPoint*) iter->data);
		while (TRUE) {
			gboolean _tmp1_;
			if (!(iter->next != NULL)) {
				break;
			}
			_tmp1_ = FALSE;
			if (time < gpx_point_get_time ((GpxPoint*) iter->next->data)) {
				gboolean _tmp2_;
				_tmp2_ = FALSE;
				if (time == gpx_point_get_time ((GpxPoint*) iter->data)) {
					_tmp2_ = TRUE;
				} else {
					_tmp2_ = time > gpx_point_get_time ((GpxPoint*) iter->data);
				}
				_tmp1_ = _tmp2_;
			} else {
				_tmp1_ = FALSE;
			}
			if (_tmp1_) {
				g_signal_emit_by_name (self, "point-clicked", ((GpxPoint*) iter->data)->lat_dec, ((GpxPoint*) iter->data)->lon_dec);
				result = FALSE;
				return result;
			}
			iter = iter->next;
		}
	}
	result = FALSE;
	return result;
}


static void gpx_graph_size_allocate_cb (GpxGraph* self, const GdkRectangle* alloc) {
	cairo_surface_t* _tmp0_;
	g_return_if_fail (self != NULL);
	/* Invalidate the previous plot, so it is redrawn */
	_tmp0_ = NULL;
	self->priv->surf = (_tmp0_ = NULL, (self->priv->surf == NULL) ? NULL : (self->priv->surf = (cairo_surface_destroy (self->priv->surf), NULL)), _tmp0_);
}


static gboolean gpx_graph_real_expose_event (GtkWidget* base, const GdkEventExpose* event) {
	GpxGraph * self;
	gboolean result;
	cairo_t* ctx;
	self = (GpxGraph*) base;
	ctx = gdk_cairo_create ((GdkDrawable*) ((GtkWidget*) self)->window);
	/* If no valid surface, render it */
	if (self->priv->surf == NULL) {
		gpx_graph_update_surface (self, (GtkWidget*) self);
	}
	/* Draw the actual surface on the widget */
	cairo_set_source_surface (ctx, self->priv->surf, (double) 0, (double) 0);
	gdk_cairo_region (ctx, (*event).region);
	cairo_clip (ctx);
	cairo_paint (ctx);
	result = FALSE;
	(ctx == NULL) ? NULL : (ctx = (cairo_destroy (ctx), NULL));
	return result;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


static void gpx_graph_update_surface (GpxGraph* self, GtkWidget* win) {
	cairo_t* ctx;
	cairo_surface_t* _tmp0_;
	cairo_t* _tmp1_;
	double max_speed;
	double elapsed_time;
	GpxPoint* _tmp5_;
	GpxPoint* f;
	double graph_width;
	double graph_height;
	PangoLayout* _tmp6_;
	PangoLayout* layout;
	double j;
	double step_size;
	char* _tmp8_;
	double _tmp9_;
	double size;
	GList* iter;
	guint interval;
	gint current;
	guint i;
	double avg;
	time_t moving_time = {0};
	g_return_if_fail (self != NULL);
	g_return_if_fail (win != NULL);
	ctx = gdk_cairo_create ((GdkDrawable*) win->window);
	_tmp0_ = NULL;
	self->priv->surf = (_tmp0_ = cairo_surface_create_similar (cairo_get_target (ctx), CAIRO_CONTENT_COLOR_ALPHA, win->allocation.width, win->allocation.height), (self->priv->surf == NULL) ? NULL : (self->priv->surf = (cairo_surface_destroy (self->priv->surf), NULL)), _tmp0_);
	_tmp1_ = NULL;
	ctx = (_tmp1_ = cairo_create (self->priv->surf), (ctx == NULL) ? NULL : (ctx = (cairo_destroy (ctx), NULL)), _tmp1_);
	/* Paint background white */
	cairo_set_source_rgba (ctx, (double) 1, (double) 1, (double) 1, (double) 1);
	cairo_paint (ctx);
	if (self->priv->track == NULL) {
		(ctx == NULL) ? NULL : (ctx = (cairo_destroy (ctx), NULL));
		return;
	}
	max_speed = (double) 0;
	if (gpx_graph_get_smooth_factor (self) != 1) {
		GList* iter;
		iter = g_list_first (self->priv->track->points);
		while (TRUE) {
			GList* ii;
			double speed;
			gint i;
			gint sf;
			double _tmp4_;
			if (!(iter->next != NULL)) {
				break;
			}
			ii = iter->next;
			speed = (double) 0;
			i = 0;
			sf = gpx_graph_get_smooth_factor (self);
			{
				gboolean _tmp2_;
				i = 0;
				_tmp2_ = TRUE;
				while (TRUE) {
					gboolean _tmp3_;
					if (!_tmp2_) {
						i++;
					}
					_tmp2_ = FALSE;
					_tmp3_ = FALSE;
					if (i < sf) {
						_tmp3_ = ii->prev != NULL;
					} else {
						_tmp3_ = FALSE;
					}
					if (!_tmp3_) {
						break;
					}
					speed = speed + gpx_track_calculate_point_to_point_speed (self->priv->track, (GpxPoint*) ii->prev->data, (GpxPoint*) ii->data);
					ii = ii->prev;
				}
			}
			speed = speed / i;
			_tmp4_ = 0.0;
			if (speed > max_speed) {
				_tmp4_ = speed;
			} else {
				_tmp4_ = max_speed;
			}
			max_speed = _tmp4_;
			iter = iter->next;
		}
	} else {
		max_speed = self->priv->track->max_speed;
	}
	elapsed_time = (double) gpx_track_get_total_time (self->priv->track);
	cairo_translate (ctx, (double) self->priv->LEFT_OFFSET, (double) 20);
	_tmp5_ = NULL;
	f = (_tmp5_ = (GpxPoint*) self->priv->track->points->data, (_tmp5_ == NULL) ? NULL : gpx_point_ref (_tmp5_));
	/* Draw Grid */
	graph_width = (double) ((win->allocation.width - self->priv->LEFT_OFFSET) - 10);
	graph_height = (double) ((win->allocation.height - 20) - self->priv->BOTTOM_OFFSET);
	_tmp6_ = NULL;
	layout = (_tmp6_ = pango_cairo_create_layout (ctx), (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_));
	j = 0.0;
	step_size = graph_height / 8.0;
	cairo_set_source_rgba (ctx, 0.2, 0.2, 0.2, 0.6);
	cairo_set_line_width (ctx, (double) 1);
	{
		gboolean _tmp7_;
		j = graph_height;
		_tmp7_ = TRUE;
		while (TRUE) {
			if (!_tmp7_) {
				j = j - step_size;
			}
			_tmp7_ = FALSE;
			if (!(j > 0.0)) {
				break;
			}
			cairo_move_to (ctx, 0.0, j);
			cairo_line_to (ctx, graph_width, j);
			cairo_stroke (ctx);
		}
	}
	/* Draw speed and ticks */
	cairo_set_source_rgba (ctx, 0.0, 0.0, 0.0, 1.0);
	_tmp8_ = NULL;
	size = (_tmp9_ = (double) (self->priv->LEFT_OFFSET / string_get_length (_tmp8_ = g_strdup_printf ("%.1f", max_speed))), _tmp8_ = (g_free (_tmp8_), NULL), _tmp9_);
	if (size > step_size) {
		size = step_size;
	}
	pango_font_description_set_absolute_size (self->priv->fd, size * 1024);
	pango_layout_set_font_description (layout, self->priv->fd);
	{
		gboolean _tmp10_;
		j = (double) 0;
		_tmp10_ = TRUE;
		while (TRUE) {
			double speed;
			char* text;
			gint w;
			gint h;
			if (!_tmp10_) {
				j = j + step_size;
			}
			_tmp10_ = FALSE;
			if (!(j < graph_height)) {
				break;
			}
			speed = max_speed * ((graph_height - j) / graph_height);
			text = g_strdup_printf ("%.1f", speed);
			w = 0;
			h = 0;
			pango_layout_set_text (layout, text, -1);
			pango_layout_get_pixel_size (layout, &w, &h);
			cairo_move_to (ctx, (double) ((-w) - 5), j - (h / 2.0));
			pango_cairo_layout_path (ctx, layout);
			cairo_fill (ctx);
			cairo_move_to (ctx, (double) (-4), j);
			cairo_line_to (ctx, (double) 0, j);
			cairo_stroke (ctx);
			text = (g_free (text), NULL);
		}
	}
	/* Draw axis */
	cairo_move_to (ctx, 0.0, 0.0);
	cairo_set_source_rgba (ctx, 0.0, 0.0, 0.0, (double) 1);
	cairo_set_line_width (ctx, 1.5);
	cairo_line_to (ctx, 0.0, graph_height);
	cairo_line_to (ctx, graph_width, graph_height);
	cairo_stroke (ctx);
	/* Draw the graph */
	cairo_set_source_rgba (ctx, 0.1, 0.2, 0.3, (double) 1);
	cairo_set_line_width (ctx, (double) 1);
	iter = g_list_first (self->priv->track->points);
	cairo_move_to (ctx, 0.0, graph_height);
	while (TRUE) {
		double time_offset;
		double speed;
		GList* ii;
		gint i;
		gint sf;
		if (!(iter->next != NULL)) {
			break;
		}
		time_offset = (double) (gpx_point_get_time ((GpxPoint*) iter->data) - gpx_point_get_time (f));
		speed = (double) 0;
		ii = iter->next;
		i = 0;
		sf = gpx_graph_get_smooth_factor (self);
		{
			gboolean _tmp11_;
			i = 0;
			_tmp11_ = TRUE;
			while (TRUE) {
				gboolean _tmp12_;
				if (!_tmp11_) {
					i++;
				}
				_tmp11_ = FALSE;
				_tmp12_ = FALSE;
				if (i < sf) {
					_tmp12_ = ii->prev != NULL;
				} else {
					_tmp12_ = FALSE;
				}
				if (!_tmp12_) {
					break;
				}
				speed = speed + gpx_track_calculate_point_to_point_speed (self->priv->track, (GpxPoint*) ii->prev->data, (GpxPoint*) ii->data);
				ii = ii->prev;
			}
		}
		speed = speed / i;
		cairo_line_to (ctx, graph_width * ((double) (time_offset / ((double) elapsed_time))), graph_height * ((double) (1.0 - (speed / max_speed))));
		iter = iter->next;
	}
	cairo_line_to (ctx, graph_width, graph_height);
	cairo_close_path (ctx);
	cairo_stroke_preserve (ctx);
	cairo_set_source_rgba (ctx, 0.1, 0.2, 0.8, 0.5);
	cairo_fill (ctx);
	iter = g_list_first (self->priv->track->points);
	cairo_set_source_rgba (ctx, 0.0, 0.0, 0.0, 1.0);
	pango_font_description_set_absolute_size (self->priv->fd, (double) (12 * 1024));
	pango_layout_set_font_description (layout, self->priv->fd);
	interval = ((guint) elapsed_time) / ((guint) (graph_width / (5 * 12.0)));
	current = 0;
	i = 0U;
	{
		gboolean _tmp13_;
		i = (guint) 0;
		_tmp13_ = TRUE;
		while (TRUE) {
			if (!_tmp13_) {
				i = i + interval;
			}
			_tmp13_ = FALSE;
			if (!(i < elapsed_time)) {
				break;
			}
			if ((graph_width * (1 - (i / elapsed_time))) > (2.5 * 12)) {
				gint w;
				gint h;
				char* text;
				w = 0;
				h = 0;
				text = g_strdup_printf ("%02i:%02i", ((gint) i) / 3600, (((gint) i) % 3600) / 60);
				pango_layout_set_text (layout, text, -1);
				pango_layout_get_pixel_size (layout, &w, &h);
				cairo_move_to (ctx, (graph_width * ((double) (i / elapsed_time))) - (w / 2.0), graph_height + 10);
				pango_cairo_layout_path (ctx, layout);
				cairo_fill (ctx);
				cairo_move_to (ctx, graph_width * ((double) (i / elapsed_time)), graph_height);
				cairo_line_to (ctx, graph_width * ((double) (i / elapsed_time)), graph_height + 5);
				cairo_stroke (ctx);
				text = (g_free (text), NULL);
			}
			current++;
		}
	}
	/* Draw average speed */
	avg = gpx_track_get_track_average (self->priv->track);
	cairo_set_source_rgba (ctx, 0.0, 0.7, 0.0, 0.7);
	cairo_move_to (ctx, 0.0, graph_height * (1 - (avg / max_speed)));
	cairo_line_to (ctx, graph_width, graph_height * (1 - (avg / max_speed)));
	cairo_stroke (ctx);
	/* Draw moving speed */
	avg = gpx_track_calculate_moving_average (self->priv->track, &moving_time);
	cairo_set_source_rgba (ctx, 0.7, 0.0, 0.0, 0.7);
	cairo_move_to (ctx, 0.0, graph_height * (1 - (avg / max_speed)));
	cairo_line_to (ctx, graph_width, graph_height * (1 - (avg / max_speed)));
	cairo_stroke (ctx);
	{
		gint w;
		gint h;
		w = 0;
		h = 0;
		cairo_set_source_rgba (ctx, 0.0, 0.0, 0.0, 1.0);
		pango_font_description_set_absolute_size (self->priv->fd, (double) (12 * 1024));
		pango_layout_set_font_description (layout, self->priv->fd);
		if (gpx_graph_get_smooth_factor (self) != 1) {
			char* _tmp14_;
			_tmp14_ = NULL;
			pango_layout_set_markup (layout, _tmp14_ = g_strdup_printf ("Speed (km/h) vs Time (HH:MM) <i>(smooth window: %i)</i>", gpx_graph_get_smooth_factor (self)), -1);
			_tmp14_ = (g_free (_tmp14_), NULL);
		} else {
			pango_layout_set_text (layout, "Speed (km/h) vs Time (HH:MM)", -1);
		}
		pango_layout_get_pixel_size (layout, &w, &h);
		cairo_move_to (ctx, (graph_width / 2) - (w / 2), (double) (-20));
		pango_cairo_layout_path (ctx, layout);
		cairo_fill (ctx);
	}
	(ctx == NULL) ? NULL : (ctx = (cairo_destroy (ctx), NULL));
	(f == NULL) ? NULL : (f = (gpx_point_unref (f), NULL));
	(layout == NULL) ? NULL : (layout = (g_object_unref (layout), NULL));
}


gint gpx_graph_get_smooth_factor (GpxGraph* self) {
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_smooth_factor;
	return result;
}


void gpx_graph_set_smooth_factor (GpxGraph* self, gint value) {
	cairo_surface_t* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_smooth_factor = value;
	/* Invalidate the previous plot, so it is redrawn */
	_tmp0_ = NULL;
	self->priv->surf = (_tmp0_ = NULL, (self->priv->surf == NULL) ? NULL : (self->priv->surf = (cairo_surface_destroy (self->priv->surf), NULL)), _tmp0_);
	/* Force a redraw */
	gtk_widget_queue_draw ((GtkWidget*) self);
	g_object_notify ((GObject *) self, "smooth-factor");
}


static void gpx_graph_class_init (GpxGraphClass * klass) {
	gpx_graph_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GpxGraphPrivate));
	GTK_WIDGET_CLASS (klass)->expose_event = gpx_graph_real_expose_event;
	G_OBJECT_CLASS (klass)->get_property = gpx_graph_get_property;
	G_OBJECT_CLASS (klass)->set_property = gpx_graph_set_property;
	G_OBJECT_CLASS (klass)->finalize = gpx_graph_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GPX_GRAPH_SMOOTH_FACTOR, g_param_spec_int ("smooth-factor", "smooth-factor", "smooth-factor", G_MININT, G_MAXINT, 4, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_signal_new ("point_clicked", GPX_TYPE_GRAPH, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__DOUBLE_DOUBLE, G_TYPE_NONE, 2, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
}


static void gpx_graph_instance_init (GpxGraph * self) {
	self->priv = GPX_GRAPH_GET_PRIVATE (self);
	self->priv->_smooth_factor = 4;
	self->priv->track = NULL;
	self->priv->fd = NULL;
	self->priv->surf = NULL;
	self->priv->LEFT_OFFSET = 60;
	self->priv->BOTTOM_OFFSET = 30;
}


static void gpx_graph_finalize (GObject* obj) {
	GpxGraph * self;
	self = GPX_GRAPH (obj);
	(self->priv->track == NULL) ? NULL : (self->priv->track = (g_object_unref (self->priv->track), NULL));
	(self->priv->fd == NULL) ? NULL : (self->priv->fd = (pango_font_description_free (self->priv->fd), NULL));
	(self->priv->surf == NULL) ? NULL : (self->priv->surf = (cairo_surface_destroy (self->priv->surf), NULL));
	G_OBJECT_CLASS (gpx_graph_parent_class)->finalize (obj);
}


GType gpx_graph_get_type (void) {
	static GType gpx_graph_type_id = 0;
	if (gpx_graph_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GpxGraphClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gpx_graph_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GpxGraph), 0, (GInstanceInitFunc) gpx_graph_instance_init, NULL };
		gpx_graph_type_id = g_type_register_static (GTK_TYPE_EVENT_BOX, "GpxGraph", &g_define_type_info, 0);
	}
	return gpx_graph_type_id;
}


static void gpx_graph_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GpxGraph * self;
	gpointer boxed;
	self = GPX_GRAPH (object);
	switch (property_id) {
		case GPX_GRAPH_SMOOTH_FACTOR:
		g_value_set_int (value, gpx_graph_get_smooth_factor (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gpx_graph_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GpxGraph * self;
	self = GPX_GRAPH (object);
	switch (property_id) {
		case GPX_GRAPH_SMOOTH_FACTOR:
		gpx_graph_set_smooth_factor (self, g_value_get_int (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}



static void g_cclosure_user_marshal_VOID__DOUBLE_DOUBLE (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__DOUBLE_DOUBLE) (gpointer data1, double arg_1, double arg_2, gpointer data2);
	register GMarshalFunc_VOID__DOUBLE_DOUBLE callback;
	register GCClosure * cc;
	register gpointer data1, data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 3);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__DOUBLE_DOUBLE) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_double (param_values + 1), g_value_get_double (param_values + 2), data2);
}



